@@ -1,9 +1,13 @@
+# frozen_string_literal: true
+
 require 'rack/auth/basic'
 
 module Grape
   module Middleware
     module Auth
       class Base
+        include Helpers
+
         attr_accessor :options, :app, :env
 
         def initialize(app, **options)
@@ -11,10 +15,6 @@ module Grape
           @options = options
         end
 
-        def context
-          env[Grape::Env::API_ENDPOINT]
-        end
-
         def call(env)
           dup._call(env)
         end
