@@ -1,14 +1,15 @@
+# frozen_string_literal: true
+
 module Grape
   module Validations
     class AsValidator < Base
       def initialize(attrs, options, required, scope, opts = {})
-        @alias = options
+        @renamed_options = options
         super
       end
 
       def validate_param!(attr_name, params)
-        params[@alias] = params[attr_name]
-        params.delete(attr_name)
+        params[@renamed_options] = params[attr_name]
       end
     end
   end
