@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'spec_helper'
 
 describe Grape::Endpoint do
@@ -8,7 +10,7 @@ describe Grape::Endpoint do
   end
 
   describe '.before_each' do
-    after { Grape::Endpoint.before_each(nil) }
+    after { Grape::Endpoint.before_each.clear }
 
     it 'is settable via block' do
       block = ->(_endpoint) { 'noop' }
@@ -149,7 +151,7 @@ describe Grape::Endpoint do
     it 'includes headers passed as symbols' do
       env = Rack::MockRequest.env_for('/headers')
       env['HTTP_SYMBOL_HEADER'.to_sym] = 'Goliath passes symbols'
-      body = subject.call(env)[2].body.first
+      body = read_chunks(subject.call(env)[2]).join
       expect(JSON.parse(body)['Symbol-Header']).to eq('Goliath passes symbols')
     end
   end
@@ -294,9 +296,12 @@ describe Grape::Endpoint do
               optional :seventh
             end
           end
+          optional :nested_arr, type: Array do
+            optional :eighth
+          end
         end
-        optional :nested_arr, type: Array do
-          optional :eighth
+        optional :arr, type: Array do
+          optional :nineth
         end
       end
     end
@@ -388,7 +393,7 @@ describe Grape::Endpoint do
 
       get '/declared?first=present&nested[fourth]=1'
       expect(last_response.status).to eq(200)
-      expect(JSON.parse(last_response.body)['nested'].keys.size).to eq 3
+      expect(JSON.parse(last_response.body)['nested'].keys.size).to eq 4
     end
 
     it 'builds nested params when given array' do
@@ -419,7 +424,7 @@ describe Grape::Endpoint do
 
         get '/declared?first=present'
         expect(last_response.status).to eq(200)
-        expect(JSON.parse(last_response.body)['nested']).to be_a(Hash)
+        expect(JSON.parse(last_response.body)['nested']).to eq({})
       end
 
       it 'to be an array when include_missing is true' do
@@ -429,7 +434,17 @@ describe Grape::Endpoint do
 
         get '/declared?first=present'
         expect(last_response.status).to eq(200)
-        expect(JSON.parse(last_response.body)['nested_arr']).to be_a(Array)
+        expect(JSON.parse(last_response.body)['arr']).to be_a(Array)
+      end
+
+      it 'to be an array when nested and include_missing is true' do
+        subject.get '/declared' do
+          declared(params, include_missing: true)
+        end
+
+        get '/declared?first=present&nested[fourth]=1'
+        expect(last_response.status).to eq(200)
+        expect(JSON.parse(last_response.body)['nested']['nested_arr']).to be_a(Array)
       end
 
       it 'to be nil when include_missing is false' do
@@ -472,12 +487,12 @@ describe Grape::Endpoint do
       expect(last_response.status).to eq(200)
     end
 
-    it 'does not include aliased missing attributes if that option is passed' do
+    it 'does not include renamed missing attributes if that option is passed' do
       subject.params do
-        optional :aliased_original, as: :aliased
+        optional :renamed_original, as: :renamed
       end
       subject.get '/declared' do
-        error! 'expected nil', 400 if declared(params, include_missing: false).key?(:aliased)
+        error! 'expected nil', 400 if declared(params, include_missing: false).key?(:renamed)
         ''
       end
 
@@ -1089,6 +1104,36 @@ describe Grape::Endpoint do
       expect(last_response.headers['X-Custom']).to eq('value')
     end
 
+    it 'merges additional headers with headers set before call' do
+      subject.before do
+        header 'X-Before-Test', 'before-sample'
+      end
+
+      subject.get '/hey' do
+        header 'X-Test', 'test-sample'
+        error!({ 'dude' => 'rad' }, 403, 'X-Error' => 'error')
+      end
+
+      get '/hey.json'
+      expect(last_response.headers['X-Before-Test']).to eq('before-sample')
+      expect(last_response.headers['X-Test']).to eq('test-sample')
+      expect(last_response.headers['X-Error']).to eq('error')
+    end
+
+    it 'does not merges additional headers with headers set after call' do
+      subject.after do
+        header 'X-After-Test', 'after-sample'
+      end
+
+      subject.get '/hey' do
+        error!({ 'dude' => 'rad' }, 403, 'X-Error' => 'error')
+      end
+
+      get '/hey.json'
+      expect(last_response.headers['X-Error']).to eq('error')
+      expect(last_response.headers['X-After-Test']).to be_nil
+    end
+
     it 'sets the status code for the endpoint' do
       memoized_endpoint = nil
 
@@ -1492,6 +1537,9 @@ describe Grape::Endpoint do
         have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :after }),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
+                                                                       filters: [],
+                                                                       type: :finally }),
         have_attributes(name: 'endpoint_run.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                env: an_instance_of(Hash) }),
         have_attributes(name: 'format_response.grape', payload: { env: an_instance_of(Hash),
@@ -1518,6 +1566,9 @@ describe Grape::Endpoint do
         have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :after }),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
+                                                                       filters: [],
+                                                                       type: :finally }),
         have_attributes(name: 'format_response.grape', payload: { env: an_instance_of(Hash),
                                                                   formatter: a_kind_of(Module) })
       )
