@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'spec_helper'
 
 describe Grape::Middleware::Base do
@@ -30,7 +32,7 @@ describe Grape::Middleware::Base do
   end
 
   context 'callbacks on error' do
-    let(:blank_app) { ->(_) { fail StandardError } }
+    let(:blank_app) { ->(_) { raise StandardError } }
 
     it 'calls #after' do
       expect(subject).to receive(:after)
@@ -114,6 +116,14 @@ describe Grape::Middleware::Base do
     end
   end
 
+  describe '#context' do
+    subject { Grape::Middleware::Base.new(blank_app) }
+    it 'allows access to response context' do
+      subject.call(Grape::Env::API_ENDPOINT => { header: 'some header' })
+      expect(subject.context).to eq(header: 'some header')
+    end
+  end
+
   context 'options' do
     it 'persists options passed at initialization' do
       expect(Grape::Middleware::Base.new(blank_app, abc: true).options[:abc]).to be true
