@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'spec_helper'
 
 describe Grape::Middleware::Formatter do
@@ -41,7 +43,7 @@ describe Grape::Middleware::Formatter do
     end
 
     context 'xml' do
-      let(:body) { 'string' }
+      let(:body) { +'string' }
       it 'calls #to_xml if the content type is xml' do
         body.instance_eval do
           def to_xml
@@ -194,26 +196,32 @@ describe Grape::Middleware::Formatter do
       subject.options[:content_types][:custom] = "don't care"
       subject.options[:formatters][:custom] = ->(_obj, _env) { 'CUSTOM FORMAT' }
       _, _, body = subject.call('PATH_INFO' => '/info.custom')
-      expect(body.body).to eq(['CUSTOM FORMAT'])
+      expect(read_chunks(body)).to eq(['CUSTOM FORMAT'])
     end
     context 'default' do
       let(:body) { ['blah'] }
       it 'uses default json formatter' do
         _, _, body = subject.call('PATH_INFO' => '/info.json')
-        expect(body.body).to eq(['["blah"]'])
+        expect(read_chunks(body)).to eq(['["blah"]'])
       end
     end
     it 'uses custom json formatter' do
       subject.options[:formatters][:json] = ->(_obj, _env) { 'CUSTOM JSON FORMAT' }
       _, _, body = subject.call('PATH_INFO' => '/info.json')
-      expect(body.body).to eq(['CUSTOM JSON FORMAT'])
+      expect(read_chunks(body)).to eq(['CUSTOM JSON FORMAT'])
     end
   end
 
   context 'no content responses' do
     let(:no_content_response) { ->(status) { [status, {}, ['']] } }
 
-    Rack::Utils::STATUS_WITH_NO_ENTITY_BODY.each do |status|
+    STATUSES_WITHOUT_BODY = if Gem::Version.new(Rack.release) >= Gem::Version.new('2.1.0')
+                              Rack::Utils::STATUS_WITH_NO_ENTITY_BODY.keys
+                            else
+                              Rack::Utils::STATUS_WITH_NO_ENTITY_BODY
+                            end
+
+    STATUSES_WITHOUT_BODY.each do |status|
       it "does not modify a #{status} response" do
         expected_response = no_content_response[status]
         allow(app).to receive(:call).and_return(expected_response)
@@ -224,6 +232,80 @@ describe Grape::Middleware::Formatter do
 
   context 'input' do
     %w[POST PATCH PUT DELETE].each do |method|
+      context 'when body is not nil or empty' do
+        context 'when Content-Type is supported' do
+          let(:io) { StringIO.new('{"is_boolean":true,"string":"thing"}') }
+          let(:content_type) { 'application/json' }
+
+          it "parses the body from #{method} and copies values into rack.request.form_hash" do
+            subject.call(
+              'PATH_INFO' => '/info',
+              'REQUEST_METHOD' => method,
+              'CONTENT_TYPE' => content_type,
+              'rack.input' => io,
+              'CONTENT_LENGTH' => io.length
+            )
+            expect(subject.env['rack.request.form_hash']['is_boolean']).to be true
+            expect(subject.env['rack.request.form_hash']['string']).to eq('thing')
+          end
+        end
+
+        context 'when Content-Type is not supported' do
+          let(:io) { StringIO.new('{"is_boolean":true,"string":"thing"}') }
+          let(:content_type) { 'application/atom+xml' }
+
+          it 'returns a 415 HTTP error status' do
+            error = catch(:error) do
+              subject.call(
+                'PATH_INFO' => '/info',
+                'REQUEST_METHOD' => method,
+                'CONTENT_TYPE' => content_type,
+                'rack.input' => io,
+                'CONTENT_LENGTH' => io.length
+              )
+            end
+            expect(error[:status]).to eq(415)
+            expect(error[:message]).to eq("The provided content-type 'application/atom+xml' is not supported.")
+          end
+        end
+      end
+
+      context 'when body is nil' do
+        let(:io) { double }
+        before do
+          allow(io).to receive_message_chain(:rewind, :read).and_return(nil)
+        end
+
+        it 'does not read and parse the body' do
+          expect(subject).not_to receive(:read_rack_input)
+          subject.call(
+            'PATH_INFO' => '/info',
+            'REQUEST_METHOD' => method,
+            'CONTENT_TYPE' => 'application/json',
+            'rack.input' => io,
+            'CONTENT_LENGTH' => 0
+          )
+        end
+      end
+
+      context 'when body is empty' do
+        let(:io) { double }
+        before do
+          allow(io).to receive_message_chain(:rewind, :read).and_return('')
+        end
+
+        it 'does not read and parse the body' do
+          expect(subject).not_to receive(:read_rack_input)
+          subject.call(
+            'PATH_INFO' => '/info',
+            'REQUEST_METHOD' => method,
+            'CONTENT_TYPE' => 'application/json',
+            'rack.input' => io,
+            'CONTENT_LENGTH' => 0
+          )
+        end
+      end
+
       ['application/json', 'application/json; charset=utf-8'].each do |content_type|
         context content_type do
           it "parses the body from #{method} and copies values into rack.request.form_hash" do
@@ -297,12 +379,17 @@ describe Grape::Middleware::Formatter do
   end
 
   context 'send file' do
-    let(:body) { Grape::ServeFile::FileResponse.new('file') }
-    let(:app) { ->(_env) { [200, {}, body] } }
+    let(:file) { double(File) }
+    let(:file_body) { Grape::ServeStream::StreamResponse.new(file) }
+    let(:app) { ->(_env) { [200, {}, file_body] } }
 
-    it 'returns Grape::Uril::SendFileReponse' do
+    it 'returns a file response' do
+      expect(file).to receive(:each).and_yield('data')
       env = { 'PATH_INFO' => '/somewhere', 'HTTP_ACCEPT' => 'application/json' }
-      expect(subject.call(env)).to be_a(Grape::ServeFile::SendfileResponse)
+      status, headers, body = subject.call(env)
+      expect(status).to be == 200
+      expect(headers).to be == { 'Content-Type' => 'application/json' }
+      expect(read_chunks(body)).to be == ['data']
     end
   end
 
@@ -315,13 +402,17 @@ describe Grape::Middleware::Formatter do
     let(:app) { ->(_env) { [200, {}, ['']] } }
     before do
       Grape::Formatter.register :invalid, InvalidFormatter
-      Grape::ContentTypes::CONTENT_TYPES[:invalid] = 'application/x-invalid'
+      Grape::ContentTypes.register :invalid, 'application/x-invalid'
+    end
+    after do
+      Grape::ContentTypes.default_elements.delete(:invalid)
+      Grape::Formatter.default_elements.delete(:invalid)
     end
 
     it 'returns response by invalid formatter' do
       env = { 'PATH_INFO' => '/hello.invalid', 'HTTP_ACCEPT' => 'application/x-invalid' }
-      _, _, bodies = *subject.call(env)
-      expect(bodies.body.first).to eq({ message: 'invalid' }.to_json)
+      _, _, body = *subject.call(env)
+      expect(read_chunks(body).join).to eq({ message: 'invalid' }.to_json)
     end
   end
 
@@ -333,7 +424,7 @@ describe Grape::Middleware::Formatter do
         parsers: { json: ->(_object, _env) { raise StandardError, 'fail' } }
       )
       io = StringIO.new('{invalid}')
-      error = catch(:error) {
+      error = catch(:error) do
         subject.call(
           'PATH_INFO' => '/info',
           'REQUEST_METHOD' => 'POST',
@@ -341,7 +432,7 @@ describe Grape::Middleware::Formatter do
           'rack.input' => io,
           'CONTENT_LENGTH' => io.length
         )
-      }
+      end
 
       expect(error[:message]).to eq 'fail'
       expect(error[:backtrace].size).to be >= 1
