@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'spec_helper'
 module Grape
   module Util
@@ -6,10 +8,10 @@ module Grape
       subject { StackableValues.new(parent) }
 
       describe '#keys' do
-        it 'returns all key' do
+        it 'returns all keys' do
           subject[:some_thing] = :foo_bar
           subject[:some_thing_else] = :foo_bar
-          expect(subject.keys).to eq [:some_thing, :some_thing_else].sort
+          expect(subject.keys).to eq %i[some_thing some_thing_else].sort
         end
 
         it 'returns merged keys with parent' do
@@ -19,7 +21,7 @@ module Grape
           subject[:some_thing] = :foo_bar
           subject[:some_thing_more] = :foo_bar
 
-          expect(subject.keys).to eq [:some_thing, :some_thing_else, :some_thing_more].sort
+          expect(subject.keys).to eq %i[some_thing some_thing_else some_thing_more].sort
         end
       end
 
@@ -52,7 +54,7 @@ module Grape
         it 'combines parent and actual values' do
           parent[:some_thing] = :foo
           subject[:some_thing] = :foo_bar
-          expect(subject[:some_thing]).to eq [:foo, :foo_bar]
+          expect(subject[:some_thing]).to eq %i[foo foo_bar]
         end
 
         it 'parent values are not changed' do
@@ -71,27 +73,27 @@ module Grape
         it 'pushes further values' do
           subject[:some_thing] = :foo
           subject[:some_thing] = :bar
-          expect(subject[:some_thing]).to eq [:foo, :bar]
+          expect(subject[:some_thing]).to eq %i[foo bar]
         end
 
         it 'can handle array values' do
           subject[:some_thing] = :foo
-          subject[:some_thing] = [:bar, :more]
-          expect(subject[:some_thing]).to eq [:foo, [:bar, :more]]
+          subject[:some_thing] = %i[bar more]
+          expect(subject[:some_thing]).to eq [:foo, %i[bar more]]
 
-          parent[:some_thing_else] = [:foo, :bar]
-          subject[:some_thing_else] = [:some, :bar, :foo]
+          parent[:some_thing_else] = %i[foo bar]
+          subject[:some_thing_else] = %i[some bar foo]
 
-          expect(subject[:some_thing_else]).to eq [[:foo, :bar], [:some, :bar, :foo]]
+          expect(subject[:some_thing_else]).to eq [%i[foo bar], %i[some bar foo]]
         end
       end
 
       describe '#to_hash' do
         it 'returns a Hash representation' do
           parent[:some_thing] = :foo
-          subject[:some_thing] = [:bar, :more]
+          subject[:some_thing] = %i[bar more]
           subject[:some_thing_more] = :foo_bar
-          expect(subject.to_hash).to eq(some_thing: [:foo, [:bar, :more]], some_thing_more: [:foo_bar])
+          expect(subject.to_hash).to eq(some_thing: [:foo, %i[bar more]], some_thing_more: [:foo_bar])
         end
       end
 
@@ -103,11 +105,11 @@ module Grape
           grandchild = StackableValues.new child
 
           parent[:some_thing] = :foo
-          child[:some_thing] = [:bar, :more]
+          child[:some_thing] = %i[bar more]
           grandchild[:some_thing] = :grand_foo_bar
           grandchild[:some_thing_more] = :foo_bar
 
-          expect(grandchild.clone.to_hash).to eq(some_thing: [:foo, [:bar, :more], :grand_foo_bar], some_thing_more: [:foo_bar])
+          expect(grandchild.clone.to_hash).to eq(some_thing: [:foo, %i[bar more], :grand_foo_bar], some_thing_more: [:foo_bar])
         end
 
         context 'complex (i.e. not primitive) data types (ex. middleware, please see bug #930)' do
